// Include files
// from Gaudi
#include "GaudiAlg/GaudiAlgorithm.h"

/** @class AuditorTestAlg AuditorTestAlg.cpp
 *
 *
 *  @author Marco Clemencic
 *  @date   Apr 2, 2008
 */
class AuditorTestAlg : public GaudiAlgorithm
{
public:
  /// Standard constructor
  AuditorTestAlg( const std::string& name, ISvcLocator* pSvcLocator );

  ~AuditorTestAlg() override; ///< Destructor

  StatusCode initialize() override; ///< Algorithm initialization
  StatusCode execute() override;    ///< Algorithm execution
  StatusCode finalize() override;   ///< Algorithm finalization

protected:
private:
};

//-----------------------------------------------------------------------------
// Implementation for class : AuditorTestAlg
//
// Apr 2, 2008 : Marco Clemencic
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_COMPONENT( AuditorTestAlg )

//=============================================================================
// Standard constructor, initializes variables
//=============================================================================
AuditorTestAlg::AuditorTestAlg( const std::string& name, ISvcLocator* pSvcLocator )
    : GaudiAlgorithm( name, pSvcLocator )
{
}
//=============================================================================
// Destructor
//=============================================================================
AuditorTestAlg::~AuditorTestAlg() {}

//=============================================================================
// Initialization
//=============================================================================
StatusCode AuditorTestAlg::initialize()
{
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;              // error printed already by GaudiAlgorithm

  debug() << "==> Initialize" << endmsg;

  return StatusCode::SUCCESS;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode AuditorTestAlg::execute()
{

  debug() << "==> Execute" << endmsg;

  const IAuditor::CustomEventType evt( "loop" );
  auditorSvc()->before( evt, name() );
  for ( long i = 0; i < 1000000; ++i ) {
  }
  auditorSvc()->after( evt, name() );

  return StatusCode::SUCCESS;
}

//=============================================================================
//  Finalize
//=============================================================================
StatusCode AuditorTestAlg::finalize()
{

  debug() << "==> Finalize" << endmsg;

  return GaudiAlgorithm::finalize(); // must be called after all other actions
}

//=============================================================================
